import ctypes
from api.functions import image
from api import api, const
from api.types import ExceptionType
import color
api.register_all( image )


class Image( object ):
    ''' Represents a single image, supported by a MagickWand.'''

    def __init__( self, image = None, type = None ):
        self._wand = api.NewMagickWand()
        if image is not None: self.read( image, type )
        #else: self.create( 100, 100, type )

    def __del__( self ):
        if self._wand:
            self._wand = api.DestroyMagickWand( self._wand )

    def __copy__( self ):
        c = Image()
        if self._wand:
            c._wand = api.CloneMagickWand( self._wand )
        return c

    def _clear( self ):
        api.ClearMagickWand( self._wand )

    def read( self, image, type = None ):
        self._clear()
        if type:
            api.MagickSetFilename( self._wand, 'buffer.%s' % type )  # hint image type

        if hasattr( image, 'read' ):
            c = image.read()
            self._check_wand_error( api.MagickReadImageBlob( self._wand, c, len( c ) ) )
        else:
            self._check_wand_error( api.MagickReadImage( self._wand, image ) )

    def create( self, columns = 100, rows = 100, type = None ):
        if type:
            api.MagickSetFilename( self._wand, 'buffer.%s' % type )  # hint image type
        self._check_wand_error( api.MagickNewImage( self._wand, columns, rows, color.TRANSPARENT._wand ) )

    def _check_wand_error( self, func ):
        if not func:
            severity = ExceptionType()
            description = api.MagickGetException( self._wand, severity )
            raise Exception( description )

    def _get_format( self ):
        format = api.MagickGetImageFormat( self._wand )
        if format == '':
            return None
        else:
            return format
    def _set_format( self, value ):
        self._check_wand_error( api.MagickSetImageFormat( self._wand, value ) )
    format = property( _get_format, _set_format, None, 'The image format as a string, eg. "PNG".' )

    def _get_units( self ):
        units = api.MagickGetImageUnits( self._wand )
        if units == const.UndefinedResolution:
            return None
        else:
            return units
    def _set_units( self, value ):
        if value and value is const.PIXELS_PER_INCH or value is const.PIXELS_PER_CENTIMETER:
            self._check_wand_error( api.MagickSetImageUnits( self._wand, value ) )
        elif not value:
            self._check_wand_error( api.MagickSetImageUnits( self._wand, const.UndefinedResolution ) )
        else:
            raise ValueError( 'units must be one of None, PIXELS_PER_INCH or PIXELS_PER_CENTIMETER' )
    units = property( _get_units, _set_units, None, 'The units of resolution for this image, ie. None, PIXELS_PER_INCH, PIXELS_PER_CENTIMETER.' )

    def _get_resolution( self ):
        x = y = ctypes.c_double()
        self._check_wand_error( api.MagickGetImageResolution( self._wand, x, y ) )
        return ( x.value, y.value )
    def _set_resolution( self, value ):
        self._check_wand_error( api.MagickSetImageResolution( self._wand, value[0], value[1] ) )
    resolution = property( _get_resolution, _set_resolution, None, 'A tuple containing the image resolution as expressed in image_units.' )

    def _set_colorspace( self, colorspace ):
        self._check_wand_error( api.MagickSetImageColorspace( self._wand, colorspace ) )
    def _get_colorspace( self ):
        colorspace = api.MagickGetImageColorspace( self._wand )
        if colorspace == const.UndefinedColorspace:
            return None
        else:
            return colorspace
    colorspace = property( _get_colorspace, _set_colorspace, None, 'The image colorspace.' )

    def _get_background_color( self ):
        c = color.TRANSPARENT
        self._check_wand_error( api.MagickGetImageBackgroundColor( self._wand, c._wand ) )
        return c
    def _set_background_color( self, color ):
        self._check_wand_error( api.MagickSetImageBackgroundColor( self._wand, color._wand ) )
    background_color = property( _get_background_color, _set_background_color, None, 'The background color of the image.' )

    def _get_border_color( self ):
        c = color.TRANSPARENT
        self._check_wand_error( api.MagickGetImageBorderColor( self._wand, c._wand ) )
        return c
    def _set_border_color( self, color ):
        self._check_wand_error( api.MagickSetImageBorderColor( self._wand, color._wand ) )
    border_color = property( _get_border_color, _set_border_color, None, 'The border color of the image.' )

    def _get_compression( self ):
        compression = api.MagickGetImageCompression( self._wand )
        if compression == const.NO_COMPRESSION:
            return None
        else:
            return compression
    def _set_compression( self, compression ):
        if not compression:
            compression = const.NO_COMPRESSION
        self._check_wand_error( api.MagickSetImageCompression( self._wand, compression ) )
    compression = property( _get_compression, _set_compression, None,
        'The image compression to use, ie. None, BZIP_COMPRESSION,\
        FAX_COMPRESSION, GROUP4_COMPRESSION, JPEG_COMPRESSION,\
        LOSSLESS_JPEG_COMPRESSION, LZW_COMPRESSION, RLE_COMPRESSION and\
        ZIP_COMPRESSION' )

    def _get_compression_quality( self ):
        return api.MagickGetImageCompressionQuality( self._wand )
    def _set_compression_quality( self, quality ):
        self._check_wand_error( api.MagickSetImageCompressionQuality( self._wand, int( round( quality, 0 ) ) ) )
    compression_quality = property( _get_compression_quality, _set_compression_quality, None, 'The compression quality of the image, as a percent.' )

    def save( self, file = None ):
        ''' Saves the image to a file.  If no file is specified, the file is
            saved with the original filename.'''
        if hasattr( file, 'write' ):
            size = api.size_t()
            b = api.MagickGetImageBlob( self._wand, size )
            try:
                file.write( ''.join( [chr( b[i] ) for i in range( 0, size.value + 1 )] ) )
            finally:
                api.MagickRelinquishMemory( b )
        else:
            self._check_wand_error( api.MagickWriteImage( self._wand, file ) )

    def dump( self, type = None ):
        ''' Save the image to a buffer, if no type is specified, original type is used.

            returns image'''

        if type:
            api.MagickSetFilename( self._wand, 'buffer.%s' % type )  # hint image type

        result = ''
        size = api.size_t()
        b = api.MagickGetImageBlob( self._wand, size )
        try:
            result = ''.join( [chr( b[i] ) for i in range( 0, size.value + 1 )] )
        finally:
            api.MagickRelinquishMemory( b )

        return result

    def alpha( self, alpha_on ):
        ''' Activates alpha channel/matte for further operations.

            alpha_on - Boolean'''

        api.MagickSetImageAlphaChannel( self._wand, alpha_on )

    def select( self, size ):
        ''' Selects image of given size if multiple image are available in container.

            size - A tuple containing the size of the scaled image.

            returns True on success.'''

        width, height = size
        api.MagickResetIterator( self._wand )
        loop = True
        while loop:
            _width = api.MagickGetImageWidth( self._wand )
            _height = api.MagickGetImageHeight( self._wand )
            if _width == width and _height == height:
                return True

            loop = api.MagickNextImage( self._wand )

        api.MagickResetIterator( self._wand )
        return False

    def scale( self, size ):
        ''' Scales the size of image to the given dimensions.

            size - A tuple containing the size of the scaled image.'''
        self._check_wand_error( api.MagickScaleImage( self._wand, size[0], size[1] ) )
    def _get_size( self ):
        width = api.MagickGetImageWidth( self._wand )
        height = api.MagickGetImageHeight( self._wand )
        return ( width, height )
    size = property( _get_size, scale, None, 'A tuple containing the size of the image. Setting the size is the same as calling scale().' )

    def resize( self, size, filter = None, blur = 0 ):
        ''' Resize the image with the given filter and applying some blur.

            size - A tuple containing the size of the scaled image.
            filter - None, or one of BESSEL_FILTER, BLACKMAN_FILTER,
                     BOX_FILTER, CATROM_FILTER, CUBIC_FILTER, GAUSSIAN_FILTER,
                     HANNING_FILTER, HERMITE_FILTER, LANCZOS_FILTER,
                     MITCHELL_FILTER, POINT_FILTER, QUADRATIC_FILTER,
                     SINC_FILTER, or TRIANGLE_FILTER.
             blur - blur factor where > 1 is blurry, < 1 is sharp.  Default is
                    no blur.'''

        if not filter:
            filter = const.UndefinedFilter

        self._check_wand_error( api.MagickResizeImage( self._wand, size[0], size[1], filter, blur ) )

    def thumbnail( self, columns, rows = None, fit = True ):
        """MagickThumbnailImage() changes the size of an image to the given 
        dimensions and removes any associated profiles. 
        The goal is to produce small low cost thumbnail images 
        suited for display on the Web."""

        if rows is None: rows = columns

        # we try to fit the dimension like PIL does 
        if fit:
            width, height = self.size
            rel = float( width ) / float( height )
            bounds = float( columns ) / float( rows )
            if rel >= bounds: rows = int( columns / rel )
            else: columns = int( rows * rel )
        self._check_wand_error( api.MagickThumbnailImage( self._wand, columns, rows ) )


    def resample( self, resolution, filter = None, blur = 0 ):
        ''' Resample the image with the given filter and applying some blur.

            size - A tuple containing the resolution of the resampled image.
            filter - None, or one of BESSEL_FILTER, BLACKMAN_FILTER,
                     BOX_FILTER, CATROM_FILTER, CUBIC_FILTER, GAUSSIAN_FILTER,
                     HANNING_FILTER, HERMITE_FILTER, LANCZOS_FILTER,
                     MITCHELL_FILTER, POINT_FILTER, QUADRATIC_FILTER,
                     SINC_FILTER, or TRIANGLE_FILTER.
             blur - blur factor where > 1 is blurry, < 1 is sharp.  Default is
                    no blur.'''

        if not filter:
            filter = const.UndefinedFilter

        self._check_wand_error( api.MagickResampleImage( self._wand, resolution[0], resolution[1], filter, blur ) )

    def sharpen( self, radius, sigma, channel = None ):
        ''' Sharpen the image with a Gaussian operator of the given radius and
            standard deviation.  For best results, the radius should be larger
            than sigma.  If radius is 0, a suitable radius is automatically
            selected.  If channel is provided, the sharpen is restricted to
            that particular channel.

            radius - The radius of the Gaussian operator, in pixels, not
                     counting the center pixel.
            sigma - The standard deviation of the Gaussian operator, in pixels.
            channel - None, or one of RED_CHANNEL, CYAN_CHANNEL, GREEN_CHANNEL,
                      MAGENTA_CHANNEL, BLUE_CHANNEL, YELLOW_CHANNEL,
                      ALPHA_CHANNEL, OPACITY_CHANNEL, BLACK_CHANNEL,
                      INDEX_CHANNEL, ALL_CHANNELS.'''

        if channel:
            self._check_wand_error( api.MagickSharpenImageChannel( self._wand, channel, radius, sigma ) )
        else:
            self._check_wand_error( api.MagickSharpenImage( self._wand, radius, sigma ) )

    def blur( self, radius, sigma, channel = None ):
        ''' Blurs the image with a Gaussian operator of the given radius and
            standard deviation.  For best results, the radius should be larger
            than sigma.  If radius is 0, a suitable radius is automatically
            selected.  If channel is provided, the blur is restricted to
            that particular channel.

            radius - The radius of the Gaussian operator, in pixels, not
                     counting the center pixel.
            sigma - The standard deviation of the Gaussian operator, in pixels.
            channel - None, or one of RED_CHANNEL, CYAN_CHANNEL, GREEN_CHANNEL,
                      MAGENTA_CHANNEL, BLUE_CHANNEL, YELLOW_CHANNEL,
                      ALPHA_CHANNEL, OPACITY_CHANNEL, BLACK_CHANNEL,
                      INDEX_CHANNEL, ALL_CHANNELS.'''

        if channel:
            self._check_wand_error( api.MagickBlurImageChannel( self._wand, channel, radius, sigma ) )
        else:
            self._check_wand_error( api.MagickBlurImage( self._wand, radius, sigma ) )

    def motion_blur( self, radius, sigma, angle ):
        ''' Simulates motion blur in an image by convolving the image with a
            Gaussian operator of the given radius and standard deviation.  For best
            results, the radius should be larger than sigma.  If radius is 0, a
            suitable radius is automatically selected.

            radius - The radius of the Gaussian operator, in pixels, not
                     counting the center pixel.
            sigma - The standard deviation of the Gaussian operator, in pixels.
            angle - Apply the effect along this angle.'''
        self._check_wand_error( api.MagickMotionBlurImage( self._wand, radius, sigma, angle ) )

    def despeckle( self ):
        ''' Reduces the speckle noise in the image.'''
        self._check_wand_error( api.MagickDespeckleImage( self._wand ) )

    def edge( self, radius = 0 ):
        ''' Enhances the edges within the image, using a convolution filter of
            the given radius.  If no radius is given, a suitable one is
            automatically chosen.

            radius = The radius of the pixel neighborhood.'''

        self._check_wand_error( api.MagickEdgeImage( self._wand, radius ) )

    def enhance( self ):
        ''' Applies a digital filter which improves the quality of a noisy image.'''
        self._check_wand_error( api.MagickEnhanceImage( self._wand ) )

    def median_filter( self, radius ):
        ''' Applies a digital filter that improves the quality of a noisy
            image. Each pixel is replaced by the median in a set of neighboring
            pixels as defined by radius.

            radius - The radius of the pixel neighborhood.'''
        self._check_wand_error( api.MagickMedianFilterImage( self._wand, radius ) )

    def equalize( self ):
        ''' Equalizes the histogram of the image.'''
        self._check_wand_error( api.MagickEqualizeImage( self._wand ) )

    def clip( self ):
        ''' Clips along the first path from the 8BIM profile, if present.'''
        self._check_wand_error( api.MagickClipImage( self._wand ) )

    def chop( self, size, offset ):
        ''' Remove a region of the image and collapse the image to occupy the
            removed portion.

            size - A tuple describing the size of the image region.
            offset - A tuple describing the region offset.'''
        self._check_wand_error( api.MagickChopImage( self._wand, size[0], size[1], offset[0], offset[1] ) )

    def splice( self, size, offset ):
        ''' Splice a solid colour region into the image.  This can be reversed
            by a chop.

            size - A tuple describing the size of the image region.
            offset - A tuple describing the region offset.'''
        self._check_wand_error( api.MagickSpliceImage( self._wand, size[0], size[1], offset[0], offset[1] ) )

    def crop( self, size, offset ):
        ''' Remove a region of the image.

            size - A tuple describing the size of the image region.
            offset - A tuple describing the region offset.'''
        self._check_wand_error( api.MagickCropImage( self._wand, size[0], size[1], offset[0], offset[1] ) )

    def flip( self ):
        ''' Creates a vertical mirror image by reflecting the pixels around the
            central x-axis.'''
        self._check_wand_error( api.MagickFlipImage( self._wand ) )

    def flop( self ):
        ''' Creates a horizontal mirror image by reflecting the pixels around
            the central y-axis.'''
        self._check_wand_error( api.MagickFlopImage( self._wand ) )

    def magnify( self ):
        ''' Scales the image proportionally to twice its original size.'''
        self._check_wand_error( api.MagickMagnifyImage( self._wand ) )

    def minify( self ):
        ''' Scales the image proportionally to one-half its original size.'''
        self._check_wand_error( api.MagickMinifyImage( self._wand ) )

    def rotate( self, degrees, background = None ):
        ''' Rotate the image.

            degrees - The number of degrees to rotate the image.
            background - The Color to use for the empty area.  Default is the
                         image's background colour, if specified, or
                         transparent if not specified.'''

        if not background:
            try:
                background = self.background_color
            except:
                background = color.TRANSPARENT

            if not background:
                background = color.TRANSPARENT

        self._check_wand_error( api.MagickRotateImage( self._wand, background._wand, degrees ) )

    def normalize( self ):
        ''' Enhances the contrast of a color image by adjusting the pixels
            color to span the entire range of colors available.'''
        self._check_wand_error( api.MagickNormalizeImage( self._wand ) )

    def modulate( self, brightness = 100, saturation = 100, hue = 100 ):
        ''' Lets you control the brightness, saturation, and hue of an image.
            Hue is the percentage of absolute rotation from the current position.
            For example 50 results in a counter-clockwise rotation of 90 degrees,
            150 results in a clockwise rotation of 90 degrees, with 0 and 200 both
            resulting in a rotation of 180 degrees.

            brightness - The percent change in brightness.
            saturation - The percent change in saturation.
            hue - The percent change in hue.'''
        self._check_wand_error( api.MagickModulateImage( self._wand, brightness, saturation, hue ) )

    def adjust_levels( self, black_point, gamma, white_point, channel = None ):
        ''' Adjust the image levels by scaling the colors falling between
            specified white and black points to the full available quantum range.
            The parameters provided represent the black, mid, and white points.
            Colors darker than the black point are set to zero.  Colors
            brighter than the white point are set to the maximum quantum value.
            If a channel is given, the level adjustment is restricted to the
            specified channel.

            black_point - The darkest color in the image.
            gamma - The gamma correction to apply to the image.
            white_point - The lightest color in the image.
            channel - None, or one of RED_CHANNEL, CYAN_CHANNEL, GREEN_CHANNEL,
                      MAGENTA_CHANNEL, BLUE_CHANNEL, YELLOW_CHANNEL,
                      ALPHA_CHANNEL, OPACITY_CHANNEL, BLACK_CHANNEL,
                      INDEX_CHANNEL, ALL_CHANNELS.'''

        if channel:
            self._check_wand_error( api.MagickLevelImageChannel( self._wand, channel, black_point, gamma, white_point ) )
        else:
            self._check_wand_error( api.MagickLevelImage( self._wand, black_point, gamma, white_point ) )

    def increase_contrast( self ):
        ''' Increase the differences between lighter and darker elements of the image.'''
        self._check_wand_error( api.MagickContrastImage( self._wand, 1 ) )

    def decrease_contrast( self ):
        ''' Decrease the differences between lighter and darker elements of the image.'''
        self._check_wand_error( api.MagickContrastImage( self._wand, 0 ) )

    def gamma_correct( self, gamma, channel = None ):
        ''' Gamma correct an image.  If channel is provided, the gamma
            correction is restricted to that particular channel.  

            gamma - The level of gamma correction, typically in the range of
                    0.8 to 2.3.  If a value of 0 is given and a channel is
                    specified, the influence of that channel is reduced.
            channel - None, or one of RED_CHANNEL, CYAN_CHANNEL, GREEN_CHANNEL,
                      MAGENTA_CHANNEL, BLUE_CHANNEL, YELLOW_CHANNEL,
                      ALPHA_CHANNEL, OPACITY_CHANNEL, BLACK_CHANNEL,
                      INDEX_CHANNEL, ALL_CHANNELS.'''

        if channel:
            self._check_wand_error( api.MagickGammaImageChannel( self._wand, channel, gamma ) )
        else:
            self._check_wand_error( api.MagickGammaImage( self._wand, gamma ) )

    def composite( self, image, offset, operator = None, channel = None ):
        ''' Composite another image on top of this one.

            image - The Image to composite onto this one.
            offset - A tuple describing the offset of the composited image.

            operator - This operator affects how the composite is applied to
                       the image. The default is OVER_COMPOSITE_OP. Choose from
                       these operators:

                            MINUS_COMPOSITE_OP
                            ADD_COMPOSITE_OP
                            IN_COMPOSITE_OP
                            DIFFERENCE_COMPOSITE_OP
                            PLUS_COMPOSITE_OP
                            BUMPMAP_COMPOSITE_OP
                            ATOP_COMPOSITE_OP
                            XOR_COMPOSITE_OP
                            SUBTRACT_COMPOSITE_OP
                            OUT_COMPOSITE_OP
                            COPY_COMPOSITE_OP
                            DISPLACE_COMPOSITE_OP
                            OVER_COMPOSITE_OP

            channel - None, or one of RED_CHANNEL, CYAN_CHANNEL, GREEN_CHANNEL,
                      MAGENTA_CHANNEL, BLUE_CHANNEL, YELLOW_CHANNEL,
                      ALPHA_CHANNEL, OPACITY_CHANNEL, BLACK_CHANNEL,
                      INDEX_CHANNEL, ALL_CHANNELS.'''

        if not operator:
            operator = const.OVER_COMPOSITE_OP

        _x, _y = int( offset[0] ), int( offset[1] )

        if channel:
            self._check_wand_error( api.MagickCompositeImageChannel( self._wand, channel, image._wand, operator, _x, _y ) )
        else:
            self._check_wand_error( api.MagickCompositeImage( self._wand, image._wand, operator, _x, _y ) )

    def quantize( self, colors, colorspace = None, tree_depth = 1, dither = False, measure_error = False ):
        ''' Limit the colours present in an image to a fixed amount.

            colors - The number of colors in the new image.
            colorspace - Perform color reduction in this colorspace, defaults to the image's colorspace.
            tree_depth - Default is to choose an optimal tree depth of
                         Log4(colors).  A tree of this depth generally allows
                         the best representation of the reference image with
                         the least amount of memory and the fastest
                         computational speed. In some cases, such as an image
                         with low color dispersion (a few number of colors), a
                         value other than Log4(number_colors) is required. To
                         expand the color tree completely, use a value of 8.
            dither - If True, the image is dithered.  Defaults to False.
            measure_error - If True, measures the difference between the
                            original and quantized images as an error.
                            Defaults to False.'''

        if not colorspace:
            colorspace = self.colorspace

        self._check_wand_error( api.MagickQuantizeImage( self._wand, colors, colorspace, tree_depth, dither, measure_error ) )

    def border( self, color, width, height ):
        ''' Surraunds an image with a color defined by color.Color(color_str).'''

        self._check_wand_error( api.MagickBorderImage( self._wand, color._wand, width, height ) )

    def sepia_tone( self, threshold ):
        ''' Applies a special effect to the image, similar to the effect
            achieved in a photo darkroom by sepia toning.  Threshold ranges from 0 to
            QuantumRange and is a measure of the extent of the sepia toning.  A threshold
            of 80 is a good starting point for a reasonable tone. '''

        self._check_wand_error( api.MagickSepiaToneImage( self._wand, threshold ) )

    def reduce_noise( self, radius = 0 ):
        ''' Smooths the contours of an image while still preserving edge information. 

            The algorithm works by replacing each pixel with its neighbor
            closest in value.  A neighbor is defined by radius. Use a radius of 0 and
            reduce_noise selects a suitable radius for you.'''

        self._check_wand_error( api.MagickReduceNoiseImage( self._wand, radius ) )

    def add_noise( self, noise_type ):
        ''' Adds random noise to the image.

            noise_type: UniformNoise, GaussianNoise, MultiplicativeNoise,
                        ImpulseNoise, LaplacianNoise, or PoissonNoise.'''

        self._check_wand_error( api.MagickAddNoiseImage( self._wand, noise_type ) )

    def contrast_stretch( self, black_point, white_point ):
        ''' Enhances the contrast of a color image by adjusting the pixels
            color to span the entire range of colors available. '''

        self._check_wand_error( api.MagickContrastStretchImage( self._wand, black_point, white_point ) )

    def sigmoidal_contrast( self, sharpen, contrast, mid_point ):
        ''' Adjusts the contrast of an image with a non-linear sigmoidal
            contrast algorithm.  Increase the contrast of the image using a sigmoidal
            transfer function without saturating highlights or shadows.  Contrast indicates
            how much to increase the contrast (0 is none; 3 is typical; 20 is pushing it);
            mid_point indicates where midtones fall in the resultant image (0 is white; 50
            is middle-gray; 100 is black).  Set sharpen to True to increase the image
            contrast otherwise setting it to False the contrast is reduced. '''

        self._check_wand_error( api.MagickSigmoidalContrastImage( self._wand, sharpen, contrast, mid_point ) )

    def evaluate( self, operator, value, channel = None ):
        ''' Applies an arithmetic, relational, or logical expression to an
            image.  Use these operators to lighten or darken an image, to
            increase or decrease contrast in an image, or to produce the
            "negative" of an image.

            operator - one of: MAX_OPERATOR, MIN_OPERATOR, MULTIPLY_OPERATOR,
                       SET_OPERATOR, XOR_OPERATOR, AND_OPERATOR, ADD_OPERATOR,
                       LEFT_SHIFT_OPERATOR, RIGHT_SHIFT_OPERATOR,
                       SUBTRACT_OPERATOR, OR_OPERATOR, DIVIDE_OPERATOR.
            '''

        if channel:
            self._check_wand_error( api.MagickEvaluateImageChannel( self._wand, channel, operator, value ) )
        else:
            self._check_wand_error( api.MagickEvaluateImage( self._wand, operator, value ) )

    def unsharp_mask( self, radius, sigma, amount, threshold ):
        ''' Sharpens an image. We convolve the image with a Gaussian operator
            of the given radius and standard deviation (sigma). For reasonable
            results, radius should be larger than sigma. Use a radius of 0 and
            UnsharpMaskImage() selects a suitable radius for you.

            radius
                    The radius of the Gaussian, in pixels, not counting the
                    center pixel.

            sigma
                    The standard deviation of the Gaussian, in pixels.

            amount
                    The percentage of the difference between the original and
                    the blur image that is added back into the original.

            threshold
                    The threshold in pixels needed to apply the diffence
                    amount.'''

        self._check_wand_error( api.MagickUnsharpMaskImage( self._wand, radius, sigma, amount, threshold ) )

    def opaque_paint( self, target, fill, fuzz = 0, invert = False ):
        ''' Change any pixel which matches the target to the fill.

            target  - target color to match.
            fill    - replacement color for target.
            fuzz    - Target color tolerance. Defaults to 0.
            invert  - Paint anything which doesn't match the target. Defaults to False.'''

        self._check_wand_error( api.MagickPaintOpaqueImage( self._wand, target._wand, fill._wand, fuzz, invert ) )



from ctypes import ( c_int,
                     c_uint,
                     c_ubyte,
                     c_ushort,
                     c_double,
                     c_long,
                     c_longlong,
                     c_ulong,
                     c_ulonglong,
                     c_char,
                     c_char_p,
                     c_wchar_p,
                     c_void_p,
                     CFUNCTYPE,
                     POINTER,
                     Structure
                    )
STRING = c_char_p
WSTRING = c_wchar_p
# values for enumeration 'MapMode'
MapMode = c_int # enum
size_t = c_uint
Quantum = c_ushort
IndexPacket = Quantum
# values for enumeration 'MagickBooleanType'
MagickBooleanType = c_int # enum
# values for enumeration 'ComplianceType'
ComplianceType = c_int # enum
# values for enumeration 'ClassType'
ClassType = c_int # enum
# values for enumeration 'ColorspaceType'
ColorspaceType = c_int # enum

MagickRealType = c_double
MagickSizeType = c_ulonglong
MagickStatusType = c_uint

# values for enumeration 'VirtualPixelMethod'
VirtualPixelMethod = c_int # enum
# values for enumeration 'CompressionType'
CompressionType = c_int # enum
# values for enumeration 'MetricType'
MetricType = c_int # enum
# values for enumeration 'ChannelType'
ChannelType = c_int # enum
# values for enumeration 'CompositeOperator'
CompositeOperator = c_int # enum
# values for enumeration 'StorageType'
StorageType = c_int # enum
# values for enumeration 'MagickLayerMethod'
MagickLayerMethod = c_int # enum
ExtendedSignedIntegralType = c_longlong
ExtendedUnsignedIntegralType = c_ulonglong

__gnuc_va_list = STRING
va_list = __gnuc_va_list

# values for enumeration 'QuantumType'
QuantumType = c_int # enum
QuantumAny = c_ulong

# values for enumeration 'TimerState'
TimerState = c_int # enum

# values for enumeration 'ExceptionType'
ExceptionType = c_int # enum

# values for enumeration 'DistortImageMethod'
DistortImageMethod = c_int # enum

# values for enumeration 'AlignType'
AlignType = c_int # enum

# values for enumeration 'ClipPathUnits'
ClipPathUnits = c_int # enum

# values for enumeration 'DecorationType'
DecorationType = c_int # enum

# values for enumeration 'FillRule'
FillRule = c_int # enum

# values for enumeration 'GradientType'
GradientType = c_int # enum

# values for enumeration 'LineCap'
LineCap = c_int # enum

# values for enumeration 'LineJoin'
LineJoin = c_int # enum

# values for enumeration 'PrimitiveType'
PrimitiveType = c_int # enum

# values for enumeration 'ReferenceType'
ReferenceType = c_int # enum

# values for enumeration 'SpreadMethod'
SpreadMethod = c_int # enum

# values for enumeration 'GravityType'
GravityType = c_int # enum

# values for enumeration 'StyleType'
StyleType = c_int # enum

# values for enumeration 'StretchType'
StretchType = c_int # enum

# values for enumeration 'GeometryFlags'
GeometryFlags = c_int # enum

# values for enumeration 'AlphaChannelType'
AlphaChannelType = c_int # enum

# values for enumeration 'ImageType'
ImageType = c_int # enum

# values for enumeration 'InterlaceType'
InterlaceType = c_int # enum

# values for enumeration 'OrientationType'
OrientationType = c_int # enum

# values for enumeration 'ResolutionType'
ResolutionType = c_int # enum

# values for enumeration 'NoiseType'
NoiseType = c_int # enum

# values for enumeration 'PreviewType'
PreviewType = c_int # enum

# values for enumeration 'MagickEvaluateOperator'
MagickEvaluateOperator = c_int # enum

# values for enumeration 'TransmitType'
TransmitType = c_int # enum

# values for enumeration 'RenderingIntent'
RenderingIntent = c_int # enum

# values for enumeration 'FilterTypes'
FilterTypes = c_int # enum

# values for enumeration 'EndianType'
EndianType = c_int # enum

# values for enumeration 'DisposeType'
DisposeType = c_int # enum

# values for enumeration 'InterpolatePixelMethod'
InterpolatePixelMethod = c_int # enum

# values for enumeration 'LogEventType'
LogEventType = c_int # enum

# values for enumeration 'MagickThreadSupport'
MagickThreadSupport = c_int # enum

# values for enumeration 'MagickModuleType'
MagickModuleType = c_int # enum

# values for enumeration 'MontageMode'
MontageMode = c_int # enum

# values for enumeration 'MagickOption'
MagickOption = c_int # enum

# values for enumeration 'QuantumFormatType'
QuantumFormatType = c_int # enum

# values for enumeration 'ResourceType'
ResourceType = c_int # enum

# values for enumeration 'PathType'
PathType = c_int # enum

# values for enumeration 'ImageLayerMethod'
ImageLayerMethod = c_int # enum

# values for enumeration 'PaintMethod'
PaintMethod = c_int # enum

# values for enumeration 'RegistryType'
RegistryType = c_int # enum

MagickOffsetType = c_longlong



#------------------------------------------------------------------------------ 



#------------------------------------------------------------------------------ 
class SemaphoreInfo( Structure ): pass
SemaphoreInfo._fields_ = []
#------------------------------------------------------------------------------ 
class _Ascii85Info( Structure ): pass
_Ascii85Info._fields_ = []
Ascii85Info = _Ascii85Info
#------------------------------------------------------------------------------ 
class _MagickWand( Structure ): pass
_MagickWand._fields_ = []
MagickWand = _MagickWand
#------------------------------------------------------------------------------ 
class _PixelIterator( Structure ): pass
_PixelIterator._fields_ = []
PixelIterator = _PixelIterator
#------------------------------------------------------------------------------ 
class _DrawingWand( Structure ): pass
_DrawingWand._fields_ = []
DrawingWand = _DrawingWand
#------------------------------------------------------------------------------ 
class _SignatureInfo( Structure ): pass
_SignatureInfo._fields_ = []
SignatureInfo = _SignatureInfo
#------------------------------------------------------------------------------ 
class _SplayTreeInfo( Structure ): pass
_SplayTreeInfo._fields_ = []
SplayTreeInfo = _SplayTreeInfo
#------------------------------------------------------------------------------ 
class _ViewInfo( Structure ): pass
_ViewInfo._fields_ = []
ViewInfo = _ViewInfo
#------------------------------------------------------------------------------ 
class _LinkedListInfo( Structure ):
    pass
_LinkedListInfo._fields_ = []
LinkedListInfo = _LinkedListInfo
#------------------------------------------------------------------------------ 
class _RectangleInfo( Structure ): pass
_RectangleInfo._fields_ = [
    ( 'width', c_ulong ),
    ( 'height', c_ulong ),
    ( 'x', c_long ),
    ( 'y', c_long ),
]
RectangleInfo = _RectangleInfo
#------------------------------------------------------------------------------ 
class _ResampleFilter( Structure ): pass
_ResampleFilter._fields_ = []
ResampleFilter = _ResampleFilter
#------------------------------------------------------------------------------ 
class _MimeInfo( Structure ): pass
_MimeInfo._fields_ = []
MimeInfo = _MimeInfo
#------------------------------------------------------------------------------ 
class _LogInfo( Structure ): pass
_LogInfo._fields_ = []
LogInfo = _LogInfo
#------------------------------------------------------------------------------ 
class _BlobInfo( Structure ): pass
_BlobInfo._fields_ = []
BlobInfo = _BlobInfo
#------------------------------------------------------------------------------ 
class _HashmapInfo( Structure ): pass
_HashmapInfo._fields_ = []
HashmapInfo = _HashmapInfo
#------------------------------------------------------------------------------ 
class _IO_FILE( Structure ): pass
FILE = _IO_FILE
#------------------------------------------------------------------------------ 
class _PixelPacket( Structure ): pass
_PixelPacket._fields_ = [
    ( 'blue', Quantum ),
    ( 'green', Quantum ),
    ( 'red', Quantum ),
    ( 'opacity', Quantum ),
]
PixelPacket = _PixelPacket
#------------------------------------------------------------------------------ 
class _ThresholdMap( Structure ): pass
_ThresholdMap._fields_ = []
ThresholdMap = _ThresholdMap
#------------------------------------------------------------------------------ 
class _TokenInfo( Structure ): pass
_TokenInfo._fields_ = []
TokenInfo = _TokenInfo
#------------------------------------------------------------------------------ 
class _PixelWand( Structure ): pass
_PixelWand._fields_ = []
PixelWand = _PixelWand
#------------------------------------------------------------------------------ 
class _PrimaryInfo( Structure ): pass
_PrimaryInfo._pack_ = 4
_PrimaryInfo._fields_ = [
    ( 'x', c_double ),
    ( 'y', c_double ),
    ( 'z', c_double ),
]
PrimaryInfo = _PrimaryInfo
#------------------------------------------------------------------------------ 
class _ErrorInfo( Structure ): pass
_ErrorInfo._pack_ = 4
_ErrorInfo._fields_ = [
    ( 'mean_error_per_pixel', c_double ),
    ( 'normalized_mean_error', c_double ),
    ( 'normalized_maximum_error', c_double ),
]
ErrorInfo = _ErrorInfo
#------------------------------------------------------------------------------ 
class _ExceptionInfo( Structure ): pass
_ExceptionInfo._fields_ = [
    ( 'severity', ExceptionType ),
    ( 'error_number', c_int ),
    ( 'reason', STRING ),
    ( 'description', STRING ),
    ( 'exceptions', c_void_p ),
    ( 'relinquish', MagickBooleanType ),
    ( 'semaphore', POINTER( SemaphoreInfo ) ),
    ( 'signature', c_ulong ),
]
ExceptionInfo = _ExceptionInfo
#------------------------------------------------------------------------------ 
#------------------------------------------------------------------------------ 



#------------------------------------------------------------------------------ 
class _QuantumInfo( Structure ): pass
_QuantumInfo._pack_ = 4
_QuantumInfo._fields_ = [
    ( 'quantum', c_ulong ),
    ( 'format', QuantumFormatType ),
    ( 'minimum', c_double ),
    ( 'maximum', c_double ),
    ( 'scale', c_double ),
    ( 'pad', size_t ),
    ( 'min_is_white', MagickBooleanType ),
    ( 'pack', MagickBooleanType ),
    ( 'semaphore', POINTER( SemaphoreInfo ) ),
    ( 'signature', c_ulong ),
]
QuantumInfo = _QuantumInfo
#------------------------------------------------------------------------------ 
class _QuantizeInfo( Structure ): pass
_QuantizeInfo._fields_ = [
    ( 'number_colors', c_ulong ),
    ( 'tree_depth', c_ulong ),
    ( 'dither', MagickBooleanType ),
    ( 'colorspace', ColorspaceType ),
    ( 'measure_error', MagickBooleanType ),
    ( 'signature', c_ulong ),
]
QuantizeInfo = _QuantizeInfo
#------------------------------------------------------------------------------ 
class _StringInfo( Structure ): pass
_StringInfo._fields_ = [
    ( 'path', c_char * 4096 ),
    ( 'datum', POINTER( c_ubyte ) ),
    ( 'length', size_t ),
    ( 'signature', c_ulong ),
]
StringInfo = _StringInfo
#------------------------------------------------------------------------------ 
class _LongPixelPacket( Structure ): pass
_LongPixelPacket._fields_ = [
    ( 'red', c_ulong ),
    ( 'green', c_ulong ),
    ( 'blue', c_ulong ),
    ( 'opacity', c_ulong ),
    ( 'index', c_ulong ),
]
LongPixelPacket = _LongPixelPacket
#------------------------------------------------------------------------------ 
class _OptionInfo( Structure ): pass
_OptionInfo._fields_ = [
    ( 'mnemonic', STRING ),
    ( 'type', c_long ),
]
OptionInfo = _OptionInfo
#------------------------------------------------------------------------------ 
class _MontageInfo( Structure ): pass
_MontageInfo._pack_ = 4
_MontageInfo._fields_ = [
    ( 'geometry', STRING ),
    ( 'tile', STRING ),
    ( 'title', STRING ),
    ( 'frame', STRING ),
    ( 'texture', STRING ),
    ( 'font', STRING ),
    ( 'pointsize', c_double ),
    ( 'border_width', c_ulong ),
    ( 'shadow', MagickBooleanType ),
    ( 'fill', PixelPacket ),
    ( 'stroke', PixelPacket ),
    ( 'background_color', PixelPacket ),
    ( 'border_color', PixelPacket ),
    ( 'matte_color', PixelPacket ),
    ( 'gravity', GravityType ),
    ( 'filename', c_char * 4096 ),
    ( 'debug', MagickBooleanType ),
    ( 'signature', c_ulong ),
]
MontageInfo = _MontageInfo
#------------------------------------------------------------------------------ 
class _ModuleInfo( Structure ): pass
__time_t = c_long
time_t = __time_t
_ModuleInfo._fields_ = [
    ( 'path', STRING ),
    ( 'tag', STRING ),
    ( 'handle', c_void_p ),
    ( 'unregister_module', CFUNCTYPE( None ) ),
    ( 'register_module', CFUNCTYPE( c_ulong ) ),
    ( 'load_time', time_t ),
    ( 'stealth', MagickBooleanType ),
    ( 'previous', POINTER( _ModuleInfo ) ),
    ( 'next', POINTER( _ModuleInfo ) ),
    ( 'signature', c_ulong ),
]
ModuleInfo = _ModuleInfo
#------------------------------------------------------------------------------ 
class _MagicInfo( Structure ): pass
_MagicInfo._pack_ = 4
_MagicInfo._fields_ = [
    ( 'path', STRING ),
    ( 'name', STRING ),
    ( 'target', STRING ),
    ( 'magic', POINTER( c_ubyte ) ),
    ( 'length', size_t ),
    ( 'offset', MagickOffsetType ),
    ( 'stealth', MagickBooleanType ),
    ( 'previous', POINTER( _MagicInfo ) ),
    ( 'next', POINTER( _MagicInfo ) ),
    ( 'signature', c_ulong ),
]
MagicInfo = _MagicInfo
#------------------------------------------------------------------------------ 
class _LocaleInfo( Structure ): pass
_LocaleInfo._fields_ = [
    ( 'path', STRING ),
    ( 'tag', STRING ),
    ( 'message', STRING ),
    ( 'stealth', MagickBooleanType ),
    ( 'previous', POINTER( _LocaleInfo ) ),
    ( 'next', POINTER( _LocaleInfo ) ),
    ( 'signature', c_ulong ),
]
LocaleInfo = _LocaleInfo
#------------------------------------------------------------------------------ 
class _Timer( Structure ): pass
_Timer._pack_ = 4
_Timer._fields_ = [
    ( 'start', c_double ),
    ( 'stop', c_double ),
    ( 'total', c_double ),
]
Timer = _Timer
#------------------------------------------------------------------------------ 
class _TimerInfo( Structure ): pass
_TimerInfo._fields_ = [
    ( 'user', Timer ),
    ( 'elapsed', Timer ),
    ( 'state', TimerState ),
    ( 'signature', c_ulong ),
]
TimerInfo = _TimerInfo
#------------------------------------------------------------------------------ 
class _ProfileInfo( Structure ): pass
_ProfileInfo._fields_ = [
    ( 'name', STRING ),
    ( 'length', size_t ),
    ( 'info', POINTER( c_ubyte ) ),
    ( 'signature', c_ulong ),
]
ProfileInfo = _ProfileInfo
#------------------------------------------------------------------------------ 
class _ChromaticityInfo( Structure ): pass
_ChromaticityInfo._fields_ = [
    ( 'red_primary', PrimaryInfo ),
    ( 'green_primary', PrimaryInfo ),
    ( 'blue_primary', PrimaryInfo ),
    ( 'white_point', PrimaryInfo ),
]
ChromaticityInfo = _ChromaticityInfo
#------------------------------------------------------------------------------
MagickProgressMonitor = CFUNCTYPE( MagickBooleanType, STRING, MagickOffsetType, MagickSizeType, c_void_p )
#------------------------------------------------------------------------------ 
class _Image( Structure ): pass
Image = _Image
_Image._pack_ = 4
_Image._fields_ = [
    ( 'storage_class', ClassType ),
    ( 'colorspace', ColorspaceType ),
    ( 'compression', CompressionType ),
    ( 'quality', c_ulong ),
    ( 'orientation', OrientationType ),
    ( 'taint', MagickBooleanType ),
    ( 'matte', MagickBooleanType ),
    ( 'columns', c_ulong ),
    ( 'rows', c_ulong ),
    ( 'depth', c_ulong ),
    ( 'colors', c_ulong ),
    ( 'colormap', POINTER( PixelPacket ) ),
    ( 'background_color', PixelPacket ),
    ( 'border_color', PixelPacket ),
    ( 'matte_color', PixelPacket ),
    ( 'gamma', c_double ),
    ( 'chromaticity', ChromaticityInfo ),
    ( 'rendering_intent', RenderingIntent ),
    ( 'profiles', c_void_p ),
    ( 'units', ResolutionType ),
    ( 'montage', STRING ),
    ( 'directory', STRING ),
    ( 'geometry', STRING ),
    ( 'offset', c_long ),
    ( 'x_resolution', c_double ),
    ( 'y_resolution', c_double ),
    ( 'page', RectangleInfo ),
    ( 'extract_info', RectangleInfo ),
    ( 'tile_info', RectangleInfo ),
    ( 'bias', c_double ),
    ( 'blur', c_double ),
    ( 'fuzz', c_double ),
    ( 'filter', FilterTypes ),
    ( 'interlace', InterlaceType ),
    ( 'endian', EndianType ),
    ( 'gravity', GravityType ),
    ( 'compose', CompositeOperator ),
    ( 'dispose', DisposeType ),
    ( 'clip_mask', POINTER( _Image ) ),
    ( 'scene', c_ulong ),
    ( 'delay', c_ulong ),
    ( 'ticks_per_second', c_long ),
    ( 'iterations', c_ulong ),
    ( 'total_colors', c_ulong ),
    ( 'start_loop', c_long ),
    ( 'error', ErrorInfo ),
    ( 'timer', TimerInfo ),
    ( 'progress_monitor', MagickProgressMonitor ),
    ( 'client_data', c_void_p ),
    ( 'cache', c_void_p ),
    ( 'attributes', c_void_p ),
    ( 'ascii85', POINTER( Ascii85Info ) ),
    ( 'blob', POINTER( BlobInfo ) ),
    ( 'filename', c_char * 4096 ),
    ( 'magick_filename', c_char * 4096 ),
    ( 'magick', c_char * 4096 ),
    ( 'magick_columns', c_ulong ),
    ( 'magick_rows', c_ulong ),
    ( 'exception', ExceptionInfo ),
    ( 'debug', MagickBooleanType ),
    ( 'reference_count', c_long ),
    ( 'semaphore', POINTER( SemaphoreInfo ) ),
    ( 'color_profile', ProfileInfo ),
    ( 'iptc_profile', ProfileInfo ),
    ( 'generic_profile', POINTER( ProfileInfo ) ),
    ( 'generic_profiles', c_ulong ),
    ( 'signature', c_ulong ),
    ( 'previous', POINTER( _Image ) ),
    ( 'list', POINTER( _Image ) ),
    ( 'next', POINTER( _Image ) ),
    ( 'interpolate', InterpolatePixelMethod ),
    ( 'black_point_compensation', MagickBooleanType ),
    ( 'transparent_color', PixelPacket ),
    ( 'mask', POINTER( _Image ) ),
    ( 'tile_offset', RectangleInfo ),
    ( 'properties', c_void_p ),
    ( 'artifacts', c_void_p ),
]
#------------------------------------------------------------------------------ 
StreamHandler = CFUNCTYPE( size_t, POINTER( Image ), c_void_p, size_t )
#------------------------------------------------------------------------------ 
class _ImageInfo( Structure ): pass
ImageInfo = _ImageInfo
_ImageInfo._pack_ = 4
_ImageInfo._fields_ = [
    ( 'compression', CompressionType ),
    ( 'orientation', OrientationType ),
    ( 'temporary', MagickBooleanType ),
    ( 'adjoin', MagickBooleanType ),
    ( 'affirm', MagickBooleanType ),
    ( 'antialias', MagickBooleanType ),
    ( 'size', STRING ),
    ( 'extract', STRING ),
    ( 'page', STRING ),
    ( 'scenes', STRING ),
    ( 'scene', c_ulong ),
    ( 'number_scenes', c_ulong ),
    ( 'depth', c_ulong ),
    ( 'interlace', InterlaceType ),
    ( 'endian', EndianType ),
    ( 'units', ResolutionType ),
    ( 'quality', c_ulong ),
    ( 'sampling_factor', STRING ),
    ( 'server_name', STRING ),
    ( 'font', STRING ),
    ( 'texture', STRING ),
    ( 'density', STRING ),
    ( 'pointsize', c_double ),
    ( 'fuzz', c_double ),
    ( 'background_color', PixelPacket ),
    ( 'border_color', PixelPacket ),
    ( 'matte_color', PixelPacket ),
    ( 'dither', MagickBooleanType ),
    ( 'monochrome', MagickBooleanType ),
    ( 'colors', c_ulong ),
    ( 'colorspace', ColorspaceType ),
    ( 'type', ImageType ),
    ( 'preview_type', PreviewType ),
    ( 'group', c_long ),
    ( 'ping', MagickBooleanType ),
    ( 'verbose', MagickBooleanType ),
    ( 'view', STRING ),
    ( 'authenticate', STRING ),
    ( 'channel', ChannelType ),
    ( 'attributes', POINTER( Image ) ),
    ( 'options', c_void_p ),
    ( 'progress_monitor', MagickProgressMonitor ),
    ( 'client_data', c_void_p ),
    ( 'cache', c_void_p ),
    ( 'stream', StreamHandler ),
    ( 'file', POINTER( FILE ) ),
    ( 'blob', c_void_p ),
    ( 'length', size_t ),
    ( 'magick', c_char * 4096 ),
    ( 'unique', c_char * 4096 ),
    ( 'zero', c_char * 4096 ),
    ( 'filename', c_char * 4096 ),
    ( 'debug', MagickBooleanType ),
    ( 'tile', STRING ),
    ( 'subimage', c_ulong ),
    ( 'subrange', c_ulong ),
    ( 'pen', PixelPacket ),
    ( 'signature', c_ulong ),
    ( 'virtual_pixel_method', VirtualPixelMethod ),
    ( 'transparent_color', PixelPacket ),
    ( 'profile', c_void_p ),
]
#------------------------------------------------------------------------------ 
class _GeometryInfo( Structure ): pass
_GeometryInfo._pack_ = 4
_GeometryInfo._fields_ = [
    ( 'rho', c_double ),
    ( 'sigma', c_double ),
    ( 'xi', c_double ),
    ( 'psi', c_double ),
    ( 'chi', c_double ),
]
GeometryInfo = _GeometryInfo
#------------------------------------------------------------------------------ 
class _FrameInfo( Structure ): pass
_FrameInfo._fields_ = [
    ( 'width', c_ulong ),
    ( 'height', c_ulong ),
    ( 'x', c_long ),
    ( 'y', c_long ),
    ( 'inner_bevel', c_long ),
    ( 'outer_bevel', c_long ),
]
FrameInfo = _FrameInfo
#------------------------------------------------------------------------------ 
class _CoderInfo( Structure ): pass
_CoderInfo._fields_ = [
    ( 'path', STRING ),
    ( 'magick', STRING ),
    ( 'name', STRING ),
    ( 'stealth', MagickBooleanType ),
    ( 'previous', POINTER( _CoderInfo ) ),
    ( 'next', POINTER( _CoderInfo ) ),
    ( 'signature', c_ulong ),
]
CoderInfo = _CoderInfo
#------------------------------------------------------------------------------ 
class _DelegateInfo( Structure ): pass
_DelegateInfo._fields_ = [
    ( 'path', STRING ),
    ( 'decode', STRING ),
    ( 'encode', STRING ),
    ( 'commands', STRING ),
    ( 'mode', c_long ),
    ( 'thread_support', MagickBooleanType ),
    ( 'spawn', MagickBooleanType ),
    ( 'stealth', MagickBooleanType ),
    ( 'previous', POINTER( _DelegateInfo ) ),
    ( 'next', POINTER( _DelegateInfo ) ),
    ( 'signature', c_ulong ),
]
DelegateInfo = _DelegateInfo
#------------------------------------------------------------------------------ 
class _ImageAttribute( Structure ): pass
_ImageAttribute._fields_ = [
    ( 'key', STRING ),
    ( 'value', STRING ),
    ( 'compression', MagickBooleanType ),
    ( 'previous', POINTER( _ImageAttribute ) ),
    ( 'next', POINTER( _ImageAttribute ) ),
]
ImageAttribute = _ImageAttribute
#------------------------------------------------------------------------------ 
class _DoublePixelPacket( Structure ): pass
_DoublePixelPacket._pack_ = 4
_DoublePixelPacket._fields_ = [
    ( 'red', c_double ),
    ( 'green', c_double ),
    ( 'blue', c_double ),
    ( 'opacity', c_double ),
    ( 'index', c_double ),
]
DoublePixelPacket = _DoublePixelPacket
#------------------------------------------------------------------------------ 
class _MagickPixelPacket( Structure ): pass
_MagickPixelPacket._pack_ = 4
_MagickPixelPacket._fields_ = [
    ( 'storage_class', ClassType ),
    ( 'colorspace', ColorspaceType ),
    ( 'matte', MagickBooleanType ),
    ( 'fuzz', c_double ),
    ( 'depth', c_ulong ),
    ( 'red', MagickRealType ),
    ( 'green', MagickRealType ),
    ( 'blue', MagickRealType ),
    ( 'opacity', MagickRealType ),
    ( 'index', MagickRealType ),
]
MagickPixelPacket = _MagickPixelPacket
#------------------------------------------------------------------------------ 
class _ColorInfo( Structure ): pass
_ColorInfo._fields_ = [
    ( 'path', STRING ),
    ( 'name', STRING ),
    ( 'compliance', ComplianceType ),
    ( 'color', MagickPixelPacket ),
    ( 'stealth', MagickBooleanType ),
    ( 'previous', POINTER( _ColorInfo ) ),
    ( 'next', POINTER( _ColorInfo ) ),
    ( 'signature', c_ulong ),
]
ColorInfo = _ColorInfo
#------------------------------------------------------------------------------ 
class _ColorPacket( Structure ): pass
_ColorPacket._pack_ = 4
_ColorPacket._fields_ = [
    ( 'pixel', PixelPacket ),
    ( 'index', IndexPacket ),
    ( 'count', MagickSizeType ),
]
ColorPacket = _ColorPacket
#------------------------------------------------------------------------------
class _ConfigureInfo( Structure ): pass
_ConfigureInfo._fields_ = [
    ( 'path', STRING ),
    ( 'name', STRING ),
    ( 'value', STRING ),
    ( 'stealth', MagickBooleanType ),
    ( 'previous', POINTER( _ConfigureInfo ) ),
    ( 'next', POINTER( _ConfigureInfo ) ),
    ( 'signature', c_ulong ),
]
ConfigureInfo = _ConfigureInfo
#------------------------------------------------------------------------------ 
class _ChannelStatistics( Structure ): pass
_ChannelStatistics._pack_ = 4
_ChannelStatistics._fields_ = [
    ( 'depth', c_ulong ),
    ( 'minima', c_double ),
    ( 'maxima', c_double ),
    ( 'mean', c_double ),
    ( 'standard_deviation', c_double ),
]
ChannelStatistics = _ChannelStatistics
#------------------------------------------------------------------------------ 
class _TypeInfo( Structure ): pass
_TypeInfo._fields_ = [
    ( 'face', c_ulong ),
    ( 'path', STRING ),
    ( 'name', STRING ),
    ( 'description', STRING ),
    ( 'family', STRING ),
    ( 'style', StyleType ),
    ( 'stretch', StretchType ),
    ( 'weight', c_ulong ),
    ( 'encoding', STRING ),
    ( 'foundry', STRING ),
    ( 'format', STRING ),
    ( 'metrics', STRING ),
    ( 'glyphs', STRING ),
    ( 'stealth', MagickBooleanType ),
    ( 'previous', POINTER( _TypeInfo ) ),
    ( 'next', POINTER( _TypeInfo ) ),
    ( 'signature', c_ulong ),
]
TypeInfo = _TypeInfo
#------------------------------------------------------------------------------ 
class _XMLTreeInfo( Structure ): pass
_XMLTreeInfo._fields_ = []
XMLTreeInfo = _XMLTreeInfo
#------------------------------------------------------------------------------ 
class _XImportInfo( Structure ): pass
_XImportInfo._fields_ = [
    ( 'frame', MagickBooleanType ),
    ( 'borders', MagickBooleanType ),
    ( 'screen', MagickBooleanType ),
    ( 'descend', MagickBooleanType ),
    ( 'silent', MagickBooleanType ),
]
XImportInfo = _XImportInfo
#------------------------------------------------------------------------------ 
class _StopInfo( Structure ): pass
_StopInfo._pack_ = 4
_StopInfo._fields_ = [
    ( 'color', MagickPixelPacket ),
    ( 'offset', MagickRealType ),
]
StopInfo = _StopInfo
class _SegmentInfo( Structure ): pass
_SegmentInfo._pack_ = 4
_SegmentInfo._fields_ = [
    ( 'x1', c_double ),
    ( 'y1', c_double ),
    ( 'x2', c_double ),
    ( 'y2', c_double ),
]
SegmentInfo = _SegmentInfo
class _GradientInfo( Structure ): pass
_GradientInfo._fields_ = [
    ( 'type', GradientType ),
    ( 'bounding_box', RectangleInfo ),
    ( 'gradient_vector', SegmentInfo ),
    ( 'stops', POINTER( StopInfo ) ),
    ( 'number_stops', c_ulong ),
    ( 'spread', SpreadMethod ),
    ( 'debug', MagickBooleanType ),
    ( 'signature', c_ulong ),
]
GradientInfo = _GradientInfo
#------------------------------------------------------------------------------
class _AffineMatrix( Structure ): pass
AffineMatrix = _AffineMatrix
_AffineMatrix._pack_ = 4
_AffineMatrix._fields_ = [
    ( 'sx', c_double ),
    ( 'rx', c_double ),
    ( 'ry', c_double ),
    ( 'sy', c_double ),
    ( 'tx', c_double ),
    ( 'ty', c_double ),
]
#------------------------------------------------------------------------------ 
class _ElementReference( Structure ): pass
ElementReference = _ElementReference
_ElementReference._fields_ = [
    ( 'id', STRING ),
    ( 'type', ReferenceType ),
    ( 'gradient', GradientInfo ),
    ( 'signature', c_ulong ),
    ( 'previous', POINTER( _ElementReference ) ),
    ( 'next', POINTER( _ElementReference ) )
]
#------------------------------------------------------------------------------ 
class _DrawInfo( Structure ): pass
DrawInfo = _DrawInfo
_DrawInfo._pack_ = 4
_DrawInfo._fields_ = [
    ( 'primitive', STRING ),
    ( 'geometry', STRING ),
    ( 'viewbox', RectangleInfo ),
    ( 'affine', AffineMatrix ),
    ( 'gravity', GravityType ),
    ( 'fill', PixelPacket ),
    ( 'stroke', PixelPacket ),
    ( 'stroke_width', c_double ),
    ( 'gradient', GradientInfo ),
    ( 'fill_pattern', POINTER( Image ) ),
    ( 'tile', POINTER( Image ) ),
    ( 'stroke_pattern', POINTER( Image ) ),
    ( 'stroke_antialias', MagickBooleanType ),
    ( 'text_antialias', MagickBooleanType ),
    ( 'fill_rule', FillRule ),
    ( 'linecap', LineCap ),
    ( 'linejoin', LineJoin ),
    ( 'miterlimit', c_ulong ),
    ( 'dash_offset', c_double ),
    ( 'decorate', DecorationType ),
    ( 'compose', CompositeOperator ),
    ( 'text', STRING ),
    ( 'face', c_ulong ),
    ( 'font', STRING ),
    ( 'metrics', STRING ),
    ( 'family', STRING ),
    ( 'style', StyleType ),
    ( 'stretch', StretchType ),
    ( 'weight', c_ulong ),
    ( 'encoding', STRING ),
    ( 'pointsize', c_double ),
    ( 'density', STRING ),
    ( 'align', AlignType ),
    ( 'undercolor', PixelPacket ),
    ( 'border_color', PixelPacket ),
    ( 'server_name', STRING ),
    ( 'dash_pattern', POINTER( c_double ) ),
    ( 'clip_mask', STRING ),
    ( 'bounds', SegmentInfo ),
    ( 'clip_units', ClipPathUnits ),
    ( 'opacity', Quantum ),
    ( 'render', MagickBooleanType ),
    ( 'element_reference', ElementReference ),
    ( 'debug', MagickBooleanType ),
    ( 'signature', c_ulong ),
]
#------------------------------------------------------------------------------ 
class _PointInfo( Structure ): pass
_PointInfo._pack_ = 4
_PointInfo._fields_ = [
    ( 'x', c_double ),
    ( 'y', c_double ),
]
PointInfo = _PointInfo
#------------------------------------------------------------------------------ 
class _PrimitiveInfo( Structure ): pass
_PrimitiveInfo._fields_ = [
    ( 'point', PointInfo ),
    ( 'coordinates', c_ulong ),
    ( 'primitive', PrimitiveType ),
    ( 'method', PaintMethod ),
    ( 'text', STRING ),
]
PrimitiveInfo = _PrimitiveInfo
#------------------------------------------------------------------------------ 
class _TypeMetric( Structure ): pass
TypeMetric = _TypeMetric
_TypeMetric._pack_ = 4
_TypeMetric._fields_ = [
    ( 'pixels_per_em', PointInfo ),
    ( 'ascent', c_double ),
    ( 'descent', c_double ),
    ( 'width', c_double ),
    ( 'height', c_double ),
    ( 'max_advance', c_double ),
    ( 'underline_position', c_double ),
    ( 'underline_thickness', c_double ),
    ( 'bounds', SegmentInfo ),
    ( 'origin', PointInfo ),
]
#------------------------------------------------------------------------------ 
DrawContext = POINTER( _DrawingWand )
MonitorHandler = CFUNCTYPE( MagickBooleanType, STRING, MagickOffsetType, MagickSizeType, POINTER( ExceptionInfo ) )
ErrorHandler = CFUNCTYPE( None, ExceptionType, STRING, STRING )
FatalErrorHandler = CFUNCTYPE( None, ExceptionType, STRING, STRING )
WarningHandler = CFUNCTYPE( None, ExceptionType, STRING, STRING )
DecodeImageHandler = CFUNCTYPE( POINTER( Image ), POINTER( ImageInfo ), POINTER( ExceptionInfo ) )
EncodeImageHandler = CFUNCTYPE( MagickBooleanType, POINTER( ImageInfo ), POINTER( Image ) )
IsImageFormatHandler = CFUNCTYPE( MagickBooleanType, POINTER( c_ubyte ), size_t )
ImageFilterHandler = CFUNCTYPE( c_ulong, POINTER( POINTER( Image ) ), c_int, POINTER( STRING ), POINTER( ExceptionInfo ) )

#------------------------------------------------------------------------------ 
class _MagickInfo( Structure ): pass
_MagickInfo._fields_ = [
    ( 'name', STRING ),
    ( 'description', STRING ),
    ( 'version', STRING ),
    ( 'note', STRING ),
    ( 'module', STRING ),
    ( 'image_info', POINTER( ImageInfo ) ),
    ( 'decoder', POINTER( DecodeImageHandler ) ),
    ( 'encoder', POINTER( EncodeImageHandler ) ),
    ( 'magick', POINTER( IsImageFormatHandler ) ),
    ( 'client_data', c_void_p ),
    ( 'adjoin', MagickBooleanType ),
    ( 'raw', MagickBooleanType ),
    ( 'endian_support', MagickBooleanType ),
    ( 'blob_support', MagickBooleanType ),
    ( 'seekable_stream', MagickBooleanType ),
    ( 'thread_support', MagickStatusType ),
    ( 'stealth', MagickBooleanType ),
    ( 'previous', POINTER( _MagickInfo ) ),
    ( 'next', POINTER( _MagickInfo ) ),
    ( 'signature', c_ulong ),
]
MagickInfo = _MagickInfo



from lib import MagickWand

#===============================================================================
# API
#===============================================================================
class API( object ):
    def __init__( self, lib ):
        self.lib = lib
        self.functions = {}

    def register( self, name, restype = None, argtypes = None ):
        self.functions[name] = self.lib[ name ]
        self.functions[name].restype = restype
        self.functions[name].argtypes = argtypes
        return self.functions[name]

    def register_all( self, list ):
        for ( name, restype, argtypes ) in list:
            self.register( name, restype, argtypes )

    def __getattr__( self, name ):
        if self.functions.has_key( name ): return self.functions[name]
        else:
            raise KeyError( 'please register the function %s' % name )

api = API( MagickWand )



from api import api
from api.functions import pixel
api.register_all( pixel )

class Color( object ):
    ''' Represents a color, supported by a PixelWand. '''

    def __init__( self, color = None ):
        ''' Create a color.  The specified color can be a name or string
            representation of a color.  Acceptable values can be found in the
            ImageMagick documentation at

                http://www.imagemagick.org/script/color.php.
        '''

        self._wand = api.NewPixelWand()

        if color:
            api.PixelSetColor( self._wand, color )

    def __del__( self ):
        if self._wand and api:
            self._wand = api.DestroyPixelWand( self._wand )

    def __eq__( self, other ):
        ''' Two colors are equal only if they share the same normalised color
            and alpha values.'''
        return api.IsPixelWandSimilar( self._wand, other._wand, 0 )\
            and api.PixelGetAlpha( self._wand ) == api.PixelGetAlpha( other._wand )

    def __ne__( self, other ):
        return not self.__eq__( other )

RED = Color( 'red' )
GREEN = Color( 'green' )
BLUE = Color( 'blue' )
CYAN = Color( 'cyan' )
MAGENTA = Color( 'magenta' )
YELLOW = Color( 'yellow' )
WHITE = Color( 'white' )
BLACK = Color( 'black' )
PURPLE = Color( 'purple' )
PINK = Color( 'pink' )
BROWN = Color( 'brown' )
GRAY = Color( 'gray' )
GREY = Color( 'grey' )
TRANSPARENT = Color( 'transparent' )
